package com.bwie.auth.controller;

import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.service.AuthService;
import com.bwie.common.domain.User;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.result.Result;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@RestController
@Log4j2
public class AuthController {

    @Autowired
    private AuthService service;
    @Autowired
    private HttpServletRequest request;

    /**
     * 发送验证码
     * @param phone
     * @return
     */
    @PostMapping("/sendCode/{phone}")
    public Result<User> sendCode(@PathVariable String phone){
        log.info("功能名称：发送验证码，请求参数：{}，请求路径：{}，请求方法：{}",
                phone,request.getRequestURI(),request.getMethod());
        Result<User> userResult = service.sendCode(phone);
        log.info("功能名称：发送验证码，请求参数：{}，请求路径：{}，请求方法：{}",
                phone,request.getRequestURI(),request.getMethod());
        return userResult;
    }

    /**
     * 登录
     * @param user
     * @return
     */
    @PostMapping("/login")
    public Result<JwtResponse> login(@RequestBody User user){
        log.info("功能名称：登录，请求参数：{}，请求路径：{}，请求方法：{}",
                JSONObject.toJSONString(user),request.getRequestURI(),request.getMethod());
        Result<JwtResponse> login = service.login(user);
        log.info("功能名称：登录，请求参数：{}，请求路径：{}，请求方法：{}",
                JSONObject.toJSONString(user),request.getRequestURI(),request.getMethod());
        return login;
    }

    /**
     * 获取登陆者信息
     * @return
     */
    @GetMapping("/info")
    public Result info(){
        log.info("功能名称：获取登陆者信息，请求路径：{}，请求方法：{}",
                request.getRequestURI(),request.getMethod());
        User info = service.info();
        Result<User> success = Result.success(info);
        log.info("功能名称：获取登陆者信息，请求路径：{}，请求方法：{}，请求结果：{}",
                request.getRequestURI(),request.getMethod(),success);
        return success;
    }

    /**
     * 退出登录
     * @return
     */
    @PostMapping("/logout")
    public Result logout(){
        log.info("功能名称：退出登录，请求路径：{}，请求方法：{}",
                request.getRequestURI(),request.getMethod());
        service.logout();
        Result<Object> success = Result.success();
        log.info("功能名称：退出登录，请求路径：{}，请求方法：{}，请求结果：{}",
                request.getRequestURI(),request.getMethod(),success);
        return success;
    }
}
