package com.bwie.common.domain;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Tag(name = "Coupon",description = "券码表")
public class Coupon {

    /**
     * 主键
     */
    @Schema(
            title = "主键",
            type = "Integer",
            description = "券码主键id"
    )
    private Integer couponId;
    /**
     * 券码名称
     */
    @Schema(
            title = "券码名称",
            type = "String",
            description = "券码名称",
            defaultValue = "爱奇艺"
    )
    private String couponName;
    /**
     * 库存
     */
    @Schema(
            title = "库存",
            type = "Integer",
            description = "券码剩余数量",
            defaultValue = "12"
    )
    private Integer num;
    /**
     * 售货状态
     */
    @Schema(
            title = "售货状态",
            type = "Integer",
            description = "售货状态",
            defaultValue = "0:可预购，1:直接购买"
    )
    private Integer status;
    /**
     * 类型
     */
    @Schema(
            title = "类型",
            type = "Integer",
            description = "券码类型",
            defaultValue = "生活类"
    )
    private Integer typeId;
    /**
     * 价格
     */
    @Schema(
            title = "价格",
            type = "BigDecimal",
            description = "券码价格",
            defaultValue = "23.20"
    )
    private BigDecimal price;
    /**
     * 售货日期
     */
    @Schema(
            title = "售货日期",
            type = "Integer",
            description = "售货日期",
            defaultValue = "2024-8-12 10:39:00"
    )
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date indate;
    /**
     * 售卖链接
     */
    @Schema(
            title = "售卖链接",
            type = "String",
            description = "根据链接购买相对应的券",
            defaultValue = "947232JJDK"
    )
    private String couponLink;

    /**
     * 券码分类
     */
    @Schema(
            title = "券码分类",
            type = "String",
            description = "typeName",
            defaultValue = "爱奇艺"
    )
    private String typeName;
    /**
     * 券码类型
     */
    @Schema(
            title = "券码类型",
            type = "String",
            description = "typeCoupon",
            defaultValue = "爱奇艺"
    )
    private String typeCoupon;

}
