package com.bwie.es.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bwie.common.domain.Coupon;
import com.bwie.common.domain.request.CouponReq;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.common.utils.StringUtils;
import com.bwie.es.service.EsService;
import lombok.extern.log4j.Log4j2;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Log4j2
public class EsServiceImpl implements EsService {

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    private static final String INDEX_NAME = "coupon";


    @Override
    public void batchAdd(List<Coupon> couponList) {
        try {
            // 创建 BulkRequest
            BulkRequest bulkRequest = new BulkRequest();
            // 遍历数据 创建 IndexRequest 添加到 BulkRequest
            couponList.forEach(coupon -> {
                bulkRequest.add(new IndexRequest(INDEX_NAME)
                        .id(coupon.getCouponId()+"")
                        .source(JSONObject.toJSONString(coupon),XContentType.JSON));
            });
            // 执行请求
            restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            log.error("批量同步添加信息异常，参数：{}，异常信息：{}",
                    JSONObject.toJSONString(couponList), e);
        }
    }

    @Override
    public Result<PageResult<Coupon>> search(CouponReq couponReq) {
        long total= 0 ;
        List<Coupon> couponList = new ArrayList<>();
        try {
            SearchRequest searchRequest = new SearchRequest(INDEX_NAME);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
            if(StringUtils.isNotBlank(couponReq.getCouponName())){
                boolQueryBuilder.must(QueryBuilders.matchQuery("couponName",couponReq.getCouponName()));
            }
            searchSourceBuilder.query(boolQueryBuilder);
            searchSourceBuilder.from((couponReq.getPageNum()-1)*couponReq.getPageSize());
            searchSourceBuilder.size(couponReq.getPageSize());
            searchSourceBuilder.highlighter(
                    new HighlightBuilder()
                            .field("couponName")
                            .preTags("<font style=\"color:red;\">")
                            .postTags("</font>"));
            searchRequest.source(searchSourceBuilder);
            SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = searchResponse.getHits();
            total = hits.getTotalHits().value;
            SearchHit[] searchHits = hits.getHits();
            for (SearchHit searchHit : searchHits) {
                String sourceAsString = searchHit.getSourceAsString();
                Coupon coupon1 = JSONObject.parseObject(sourceAsString, Coupon.class);
                Map<String, HighlightField> highlightFields = searchHit.getHighlightFields();
                if(highlightFields!=null){
                    HighlightField highlightField = highlightFields.get("couponName");
                    if(highlightField!=null){
                        Text[] fragments = highlightField.getFragments();
                        StringBuilder sb = new StringBuilder();
                        for (Text fragment : fragments) {
                            sb.append(fragment);
                        }
                        coupon1.setCouponName(sb.toString());
                    }
                }
                coupon1.setCouponId(Integer.valueOf(searchHit.getId()));
                couponList.add(coupon1);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return PageResult.toResult(total,couponList);

    }

}
