package com.bwie.rabbitmq.consumer;

import cn.hutool.core.util.RandomUtil;
import com.aliyun.dysmsapi20170525.models.SendSmsResponseBody;
import com.bwie.common.domain.User;
import com.bwie.rabbitmq.remote.MqRemote;
import com.rabbitmq.client.Channel;
import com.bwie.common.utils.StringUtils;
import com.bwie.rabbitmq.utils.TelSmsUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Objects;

@Component
@Log4j2
public class SmsConsumer {
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Resource
    private MqRemote remote;
    private static final String SEND_SMS_KEY = "SEND_SMS_KEY";
    @RabbitListener(queuesToDeclare = @Queue("SEND_SMS_KEY"))
    public void sendSmsConsumer(User user, Message message, Channel channel) {
        log.info("短信队列消费者接收到消息，消息内容：{}，开始消费....", user.getPhone());
        // 获取消息的唯一标识
        String messageId = message.getMessageProperties().getMessageId();
        try {
            Long count = redisTemplate.opsForSet().add(SEND_SMS_KEY, messageId);
            if (count != null && count == 1) {

                remote.updateMoney(user.getUserId());

                // 确认消息
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                log.info("短信队列消费者接收到消息，消息内容：{}，消费完毕....", user.getPhone());
            }
        } catch (Exception e) {
            log.error("短信队列消费者接收到消息，消息内容：{}，消费消息异常，异常信息：{}", user.getPhone(), e);
            // 删除 redis 中 消息的ID
            redisTemplate.opsForSet().remove(SEND_SMS_KEY, messageId);
            // 拒绝消费消息
            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(), true);
            } catch (IOException ex) {
                log.error("短信队列消费者接收到消息，消息内容：{}，消费者拒绝消费消息异常，异常信息：{}", user.getPhone(), ex);
            }
        }
    }
}
